package controller

import (
	"github.com/gin-gonic/gin"
	"iot_manager_service/app/dao"
	"iot_manager_service/app/model"
	"iot_manager_service/app/service"
	"iot_manager_service/app/utils"
	"math"
	"net/http"
	"strconv"
)

// 灯杆基本信息管理对象
var LampPole = new(lampPoleCtl)

type lampPoleCtl struct{}

func (c *lampPoleCtl) Detail(ctx *gin.Context) {
	id, e := strconv.Atoi(ctx.Query("id"))
	if e != nil {
		ctx.JSON(http.StatusOK, utils.ParamsInvalidResponse(e.Error(), nil))
		return
	}

	device, err := service.LampPoleService.Get(id)
	if err != nil {
		ctx.JSON(http.StatusOK, err)
		return
	}
	ctx.JSON(http.StatusOK, utils.SuccessResponse(utils.Succeeded, device))
}

func (c *lampPoleCtl) CreateOrUpdate(ctx *gin.Context) {
	var req *dao.LampPole
	if err := ctx.ShouldBindJSON(&req); err != nil {
		ctx.JSON(http.StatusOK, utils.ParamsInvalidResponse(err.Error(), nil))
		return
	}
	err := service.LampPoleService.CreateOrUpdate(req)
	ctx.JSON(http.StatusOK, err)
}

func (c *lampPoleCtl) List(ctx *gin.Context) {
	searchValue := ctx.Query("searchValue")
	groupId := ctx.Query("groupId")
	boxId := ctx.Query("boxId")
	current, _ := strconv.Atoi(ctx.Query("current"))
	size, _ := strconv.Atoi(ctx.Query("size"))
	if current == 0 {
		current = 1
	}
	if size <= 0 || size > 100 {
		size = 10
	}

	devices, err := service.LampPoleService.List(searchValue, groupId, boxId, current, size)
	if err != nil {
		ctx.JSON(http.StatusOK, err)
		return
	}
	pages := math.Ceil(float64(len(devices)) / float64(size))
	rsp := model.RspLampPoleList{
		Current: current,
		Size:    size,
		Total:   len(devices),
		Pages:   int(pages),
	}
	for _, device := range devices {
		rsp.Records = append(rsp.Records, lampPoleDaoToModel(device))
	}
	ctx.JSON(http.StatusOK, utils.SuccessResponse(utils.Succeeded, rsp))
}

func (c *lampPoleCtl) GetRelevanceDetail(ctx *gin.Context) {
	id, e := strconv.Atoi(ctx.Query("id"))
	if e != nil {
		ctx.JSON(http.StatusOK, utils.ParamsInvalidResponse(e.Error(), nil))
		return
	}

	device, err := service.LampPoleService.GetRelevanceDetail(id)
	if err != nil {
		ctx.JSON(http.StatusOK, err)
		return
	}
	ctx.JSON(http.StatusOK, utils.SuccessResponse(utils.Succeeded, device))
}

func (c *lampPoleCtl) Remove(ctx *gin.Context) {
	var req *model.ReqLampPoleRemove
	if err := ctx.ShouldBindJSON(&req); err != nil {
		ctx.JSON(http.StatusOK, utils.ParamsInvalidResponse(err.Error(), nil))
		return
	}
	err := service.LampPoleService.Remove(req.IDs)
	if err != nil {
		ctx.JSON(http.StatusOK, err)
		return
	}
	ctx.JSON(http.StatusOK, utils.SuccessResponse(utils.Succeeded, nil))
}

func (c *lampPoleCtl) ImportExcel(ctx *gin.Context) {
}

func (c *lampPoleCtl) ExportExcel(ctx *gin.Context) {
}

func (c *lampPoleCtl) ExportTemplate(ctx *gin.Context) {
}

func (c *lampPoleCtl) GetList(ctx *gin.Context) {
	devices, err := service.LampPoleService.GetList()
	if err != nil {
		ctx.JSON(http.StatusOK, err)
		return
	}
	ctx.JSON(http.StatusOK, utils.SuccessResponse(utils.Succeeded, devices))
}

func lampPoleDaoToModel(device dao.LampPole) model.LampPoleDetail {
	return model.LampPoleDetail{
		LampPole:          device,
		AlarmTerminalList: nil,
		CameraList:        nil,
		CaptureUnitList:   nil,
		GatewayList:       nil,
		InfoBoardList:     nil,
		IpBroadcastList:   nil,
		LightControlList:  nil,
		SensorList:        nil,
		ZigbeeList:        nil,
	}
}
