package controller

import (
	"github.com/gin-gonic/gin"
	"iot_manager_service/app/dao"
	"iot_manager_service/app/model"
	"iot_manager_service/app/service"
	"iot_manager_service/app/utils"
	"math"
	"net/http"
	"strconv"
)

// 变压器基本信息管理对象
var Transformer = new(transformerCtl)

type transformerCtl struct{}

func (c *transformerCtl) Detail(ctx *gin.Context) {
	id, e := strconv.Atoi(ctx.Query("id"))
	if e != nil {
		ctx.JSON(http.StatusOK, utils.ParamsInvalidResponse(e.Error(), nil))
		return
	}

	device, err := service.TransformerService.Get(id)
	if err != nil {
		ctx.JSON(http.StatusOK, err)
		return
	}
	ctx.JSON(http.StatusOK, utils.SuccessResponse(utils.Succeeded, device))
}

func (c *transformerCtl) List(ctx *gin.Context) {
	searchValue := ctx.Query("searchValue")
	current, _ := strconv.Atoi(ctx.Query("current"))
	size, _ := strconv.Atoi(ctx.Query("size"))
	if current == 0 {
		current = 1
	}
	if size <= 0 || size > 100 {
		size = 10
	}
	devices, err := service.TransformerService.List(searchValue, current, size)
	if err != nil {
		ctx.JSON(http.StatusOK, err)
		return
	}
	pages := math.Ceil(float64(len(devices)) / float64(size))
	rsp := model.RsqTransformerList{
		Current: current,
		Size:    size,
		Total:   len(devices),
		Pages:   int(pages),
	}
	for _, device := range devices {

		rsp.Records = append(rsp.Records, device)
	}
	ctx.JSON(http.StatusOK, utils.SuccessResponse(utils.Succeeded, rsp))
}

func (c *transformerCtl) CreateOrUpdate(ctx *gin.Context) {
	var req dao.Transformer
	if err := ctx.ShouldBindJSON(&req); err != nil {
		ctx.JSON(http.StatusOK, utils.ParamsInvalidResponse(err.Error(), nil))
		return
	}
	err := service.TransformerService.CreateOrUpdate(req)
	ctx.JSON(http.StatusOK, err)
}

func (c *transformerCtl) Remove(ctx *gin.Context) {
	var req *model.ReqTransformerRemove
	if err := ctx.ShouldBindJSON(&req); err != nil {
		ctx.JSON(http.StatusOK, utils.ParamsInvalidResponse(err.Error(), nil))
		return
	}
	err := service.TransformerService.Remove(req.IDs)
	if err != nil {
		ctx.JSON(http.StatusOK, err)
		return
	}
	ctx.JSON(http.StatusOK, utils.SuccessResponse(utils.Succeeded, nil))
}

func (c *transformerCtl) GetList(ctx *gin.Context) {
	devices, err := service.TransformerService.GetList()
	if err != nil {
		ctx.JSON(http.StatusOK, err)
		return
	}
	ctx.JSON(http.StatusOK, utils.SuccessResponse(utils.Succeeded, devices))
}
