package service

import (
	"fmt"
	"iot_manager_service/app/dao"
	"iot_manager_service/app/model"
	"iot_manager_service/app/utils"
	"time"
)

// 中间件管理服务
var GatewayService = new(gatewayService)

type gatewayService struct{}

func (s *gatewayService) Get(id int) (*model.GatewayDetail, *utils.Errors) {
	// 创建查询实例
	device := &dao.Gateway{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}

	detail := &model.GatewayDetail{Gateway: *device}

	relation, e := GatewayRelationService.Get(id)
	if e == nil {
		detail.CountLampPole = relation.Total
	} else {
		fmt.Printf("GatewayRelationService.Get err = %v", err)
	}
	//todo 获取网关状态
	//			vo.setEndLineTime(time);
	//			vo.setNetworkState(online);
	//			vo.setRunState(online);
	return detail, nil
}

func (s *gatewayService) CreateOrUpdate(req *model.GatewayDetail) *utils.Errors {
	device := req.Gateway
	if device.TenantId == "" {
		device.TenantId = "000000" // todo: 使用登录态
	}
	device.UpdateUser = "TODO" // todo: 使用登录态
	device.UpdateTime = time.Now()
	if device.LampPoleId != 0 {
		lampPole, err := LampPoleService.GetOne(device.LampPoleId)
		if err == nil {
			device.LampLat = lampPole.PoleLat
			device.LampLng = lampPole.PoleLng
			device.LampPoleName = lampPole.PoleName
			device.LampPoleSn = lampPole.PoleSN
			device.LampPoleLocation = lampPole.InstallLocation
		} else {
			fmt.Printf("LampPoleService.GetOne err = %v \n", err)
		}
	}

	if device.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = "TODO" // todo: 使用登录态

		if device.IsExistedBySN() {
			fmt.Printf("Create IsExistedBySN \n")
			return utils.ParamsInvalidResponse(model.RepeatedPrompts, nil)
		}
		fmt.Printf("device = %+v \n", device)
		if err := device.Create(); err != nil {
			fmt.Printf("Create err = %s \n", err.Error())
			return utils.FailResponse(err.Error(), nil)
		}
		return utils.SuccessResponse(utils.Succeeded, nil)
	}

	if err := device.Update(); err != nil {
		fmt.Printf("Update err = %s \n", err.Error())
		return utils.FailResponse(err.Error(), nil)
	}

	//todo operation record
	return utils.SuccessResponse(utils.Succeeded, nil)
}

func (s *gatewayService) List(searchValue string, current, size int) ([]model.GatewayDetail, *utils.Errors) {
	var details []model.GatewayDetail
	device := dao.Gateway{}

	if searchValue != "" {
		device.GatewaySN = searchValue
	}

	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)
	for _, d := range devices {
		detail := model.GatewayDetail{Gateway: d}
		relation, _ := GatewayRelationService.Get(d.ID)
		if relation != nil {
			detail.CountLampPole = relation.Total
		}
		details = append(details, detail)
	}
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}
	return details, nil
}

func (s *gatewayService) Remove(id int) *utils.Errors {
	// 创建查询实例
	device := &dao.Gateway{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: "TODO", // todo 使用登录态
		UpdateTime: time.Now(),
	}

	if relation, _ := GatewayRelationService.Get(id); relation != nil && relation.Total > 0 {
		return utils.OperationInvalidResponse(model.GatewayHasRelation, nil)
	}

	//todo operation record
	err := device.Delete()
	if err != nil {
		return utils.FailResponse(err.Error(), nil)
	}
	return nil
}

func (s *gatewayService) GetList() ([]*dao.Gateway, *utils.Errors) {
	var devices []*dao.Gateway
	err := utils.Redis.Get(getGatewayListRedisKey("000000")).Scan(&devices)
	if err == nil {
		return devices, nil
	}

	device := &dao.Gateway{
		TenantId:  "000000", // todo 使用登录态
		IsDeleted: 0,
	}
	devices, err = device.GetAllDevices()
	for _, device := range devices {
		device.GatewayName = device.GatewayName + "(" + device.GatewaySN + ")"
	}
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}
	_ = utils.Redis.Set(getGatewayListRedisKey("000000"), devices, 0).Err()

	return devices, nil
}

func getGatewayListRedisKey(tenantId string) string {
	return fmt.Sprintf(model.WisdomGatewayList, tenantId)
}

func (s *gatewayService) GetRelevanceDetail(id int) (*model.GatewayDetail, *utils.Errors) {
	// 创建查询实例
	device := &dao.Gateway{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}
	//todo get gateway ipBroadcast lightcontroller... list
	return &model.GatewayDetail{
		Gateway:           *device,
		AlarmTerminalList: nil,
		CameraList:        nil,
		CaptureUnitList:   nil,
		InfoBoardList:     nil,
		IpBroadcastList:   nil,
		LightControlList:  nil,
		OptoSensorList:    nil,
		ZigbeeList:        nil,
	}, nil
}

func (s *gatewayService) GetOne(id int) (*dao.Gateway, *utils.Errors) {
	// 创建查询实例
	device := &dao.Gateway{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}
	return device, nil
}
