package service

import (
	"fmt"
	"iot_manager_service/app/dao"
	"iot_manager_service/app/model"
	"iot_manager_service/app/utils"
	"time"
)

// 中间件管理服务
var AlarmService = new(alarmService)

type alarmService struct{}

func (s *alarmService) Get(id int) (*dao.Alarm, *utils.Errors) {
	// 创建查询实例
	device := &dao.Alarm{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}
	return device, nil
}

func (s *alarmService) CreateOrUpdate(req dao.Alarm) *utils.Errors {
	device := req
	if device.TenantId == "" {
		device.TenantId = "000000" // todo: 使用登录态
	}
	device.UpdateUser = "TODO" // todo: 使用登录态
	device.UpdateTime = time.Now()

	if req.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = "TODO" // todo: 使用登录态

		if device.IsExistedBySN() {
			fmt.Printf("Create IsExistedBySN \n")
			return utils.ParamsInvalidResponse(model.RepeatedName, nil)
		}
		if err := device.Create(); err != nil {
			fmt.Printf("Create err = %s \n", err.Error())
			return utils.FailResponse(err.Error(), nil)
		}
		return utils.SuccessResponse(utils.Succeeded, nil)
	}

	if device.IsExistedByNameAndCode() {
		fmt.Printf("Update IsExistedByNameAndCode \n")
		return utils.ParamsInvalidResponse(model.RepeatedName, nil)
	}

	if err := device.Update(); err != nil {
		fmt.Printf("Update err = %s \n", err.Error())
		return utils.FailResponse(err.Error(), nil)
	}

	//todo operation record
	return utils.SuccessResponse(utils.Succeeded, nil)
}

func (s *alarmService) List(poleGroupName string, current, size int) ([]dao.Alarm, *utils.Errors) {
	// 创建查询实例
	device := &dao.Alarm{}
	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}
	return devices, nil
}

func (s *alarmService) Remove(id int) *utils.Errors {
	// 创建查询实例
	device := &dao.Alarm{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: "TODO", // todo 使用登录态
		UpdateTime: time.Now(),
	}

	//todo
	// service.lampPoleService.CountRelation()

	//todo operation record
	err := device.Delete()
	if err != nil {
		return utils.FailResponse(err.Error(), nil)
	}
	return nil
}

func (s *alarmService) GetList() ([]*dao.Alarm, *utils.Errors) {
	var devices []*dao.Alarm
	err := utils.Redis.Get(getAlarmListRedisKey("000000")).Scan(&devices)
	if err == nil {
		return devices, nil
	}
	device := &dao.Alarm{
		TenantId:  "000000", // todo 使用登录态
		IsDeleted: 0,
	}
	devices, err = device.GetAllDevices()
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}
	_ = utils.Redis.Set(getAlarmListRedisKey("000000"), devices, 0).Err()
	return devices, nil
}

func getAlarmListRedisKey(tenantId string) string {
	return fmt.Sprintf(model.AlarmList, tenantId)
}
