package service

import (
	"fmt"
	"iot_manager_service/app/dao"
	"iot_manager_service/app/model"
	"iot_manager_service/app/utils"
	"time"
)

// 中间件管理服务
var AlarmTerminalService = new(alarmTerminalService)

type alarmTerminalService struct{}

func (s *alarmTerminalService) Get(id int) (*dao.AlarmTerminal, *utils.Errors) {
	// 创建查询实例
	device := &dao.AlarmTerminal{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}
	return device, nil
}

func (s *alarmTerminalService) CreateOrUpdate(req dao.AlarmTerminal) *utils.Errors {
	device := req
	if device.TenantId == "" {
		device.TenantId = "000000" // todo: 使用登录态
	}
	device.UpdateUser = "TODO" // todo: 使用登录态
	device.UpdateTime = time.Now()

	if req.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = "TODO" // todo: 使用登录态

		if device.IsExistedBySN() {
			fmt.Printf("Create IsExistedBySN \n")
			return utils.ParamsInvalidResponse(model.RepeatedName, nil)
		}
		if err := device.Create(); err != nil {
			fmt.Printf("Create err = %s \n", err.Error())
			return utils.FailResponse(err.Error(), nil)
		}
		return utils.SuccessResponse(utils.Succeeded, nil)
	}

	if device.IsExistedByNameAndCode() {
		fmt.Printf("Update IsExistedByNameAndCode \n")
		return utils.ParamsInvalidResponse(model.RepeatedName, nil)
	}

	if err := device.Update(); err != nil {
		fmt.Printf("Update err = %s \n", err.Error())
		return utils.FailResponse(err.Error(), nil)
	}

	//todo operation record
	return utils.SuccessResponse(utils.Succeeded, nil)
}

func (s *alarmTerminalService) List(poleGroupName string, current, size int) ([]dao.AlarmTerminal, *utils.Errors) {
	// 创建查询实例
	device := &dao.AlarmTerminal{}
	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}
	return devices, nil
}

func (s *alarmTerminalService) Remove(id int) *utils.Errors {
	// 创建查询实例
	device := &dao.AlarmTerminal{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: "TODO", // todo 使用登录态
		UpdateTime: time.Now(),
	}

	//todo
	// service.lampPoleService.CountRelation()

	//todo operation record
	err := device.Delete()
	if err != nil {
		return utils.FailResponse(err.Error(), nil)
	}
	return nil
}
