package service

import (
	"fmt"
	"iot_manager_service/app/dao"
	"iot_manager_service/app/model"
	"iot_manager_service/app/utils"
	"time"
)

// 中间件管理服务
var CaptureUintService = new(captureUintService)

type captureUintService struct{}

func (s *captureUintService) CaptureSubmit(req *model.CaptureDetail) *utils.Errors {
	device := req.CaptureUint
	if device.TenantId == "" {
		device.TenantId = "000000" // todo: 使用登录态
	}
	device.UpdateUser = "TODO" // todo: 使用登录态
	device.UpdateTime = time.Now()
	if gateway, err := GatewayService.GetOne(device.GatewayId); err == nil {
		device.GatewayName = gateway.GatewayName
		device.GatewaySN = gateway.GatewaySN
	}

	if device.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = "TODO" // todo: 使用登录态

		if device.IsExistedBySN() {
			fmt.Printf("Create IsExistedBySN \n")
			return utils.ParamsInvalidResponse(model.RepeatedPrompts, nil)
		}
		fmt.Printf("device = %+v \n", device)
		if err := device.Create(); err != nil {
			fmt.Printf("Create err = %s \n", err.Error())
			return utils.FailResponse(err.Error(), nil)
		}
		return utils.SuccessResponse(utils.Succeeded, nil)
	}

	if err := device.Update(); err != nil {
		fmt.Printf("Update err = %s \n", err.Error())
		return utils.FailResponse(err.Error(), nil)
	}

	//todo operation record
	return utils.SuccessResponse(utils.Succeeded, nil)
}

func (s *captureUintService) GetCapture(id int) (*model.CaptureDetail, *utils.Errors) {
	// 创建查询实例
	device := &dao.CaptureUint{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}
	detail := &model.CaptureDetail{CaptureUint: *device}
	//todo 获取实时在线最新数据
	// detail.EndLineTime = *
	// detail.NetworkState = *

	return detail, nil
}

func (s *captureUintService) CaptureList(searchValue string, current, size int) ([]model.CaptureDetail, *utils.Errors) {
	var details []model.CaptureDetail
	device := dao.CaptureUint{}

	if searchValue != "" {
		device.CaptureSN = searchValue
	}

	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)

	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}

	for _, d := range devices {
		details = append(details, model.CaptureDetail{
			CaptureUint: d,
			//todo 获取实时在线最新数据
			// detail.EndLineTime = *
			// detail.NetworkState = *

		})
	}

	return details, nil
}

func (s *captureUintService) CaptureGetList() ([]*dao.CaptureUint, *utils.Errors) {
	device := &dao.CaptureUint{
		TenantId:  "000000", // todo 使用登录态
		IsDeleted: 0,
	}
	devices, err := device.GetAllDevices()
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}
	for _, d := range devices {
		d.CaptureName = d.CaptureName + "(" + d.CaptureSN + ")"
	}
	return devices, nil
}

func (s *captureUintService) GetPoint(id int) (*dao.CheckPoint, *utils.Errors) {
	// 创建查询实例
	device := &dao.CheckPoint{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}

	return device, nil
}

func (s *captureUintService) PointSubmit(req *dao.CheckPoint) *utils.Errors {
	device := req
	if device.TenantId == "" {
		device.TenantId = "000000" // todo: 使用登录态
	}
	device.UpdateUser = "TODO" // todo: 使用登录态
	device.UpdateTime = time.Now()

	if device.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = "TODO" // todo: 使用登录态

		if device.IsExistedBySN() {
			fmt.Printf("Create IsExistedBySN \n")
			return utils.ParamsInvalidResponse(model.RepeatedPrompts, nil)
		}
		fmt.Printf("device = %+v \n", device)
		if err := device.Create(); err != nil {
			fmt.Printf("Create err = %s \n", err.Error())
			return utils.FailResponse(err.Error(), nil)
		}
		return utils.SuccessResponse(utils.Succeeded, nil)
	}

	if err := device.Update(); err != nil {
		fmt.Printf("Update err = %s \n", err.Error())
		return utils.FailResponse(err.Error(), nil)
	}

	//todo operation record
	return utils.SuccessResponse(utils.Succeeded, nil)
}

func (s *captureUintService) PointList(searchValue string, current, size int) ([]dao.CheckPoint, *utils.Errors) {
	device := dao.CheckPoint{}
	if searchValue != "" {
		device.PointSN = searchValue
	}

	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}
	return devices, nil
}

func (s *captureUintService) PointGetList() ([]*dao.CheckPoint, *utils.Errors) {
	device := &dao.CheckPoint{
		TenantId:  "000000", // todo 使用登录态
		IsDeleted: 0,
	}
	devices, err := device.GetAllDevices()
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}
	for _, d := range devices {
		d.PointName = d.PointName + "(" + d.PointSN + ")"
	}
	return devices, nil
}

func (s *captureUintService) RemoveCapture(id int) *utils.Errors {
	// 创建查询实例
	device := &dao.CaptureUint{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: "TODO", // todo 使用登录态
		UpdateTime: time.Now(),
	}

	//todo operation record
	err := device.Delete()
	if err != nil {
		return utils.FailResponse(err.Error(), nil)
	}
	return nil
}

func (s *captureUintService) RemovePoint(id int) *utils.Errors {
	// 创建查询实例
	device := &dao.CheckPoint{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: "TODO", // todo 使用登录态
		UpdateTime: time.Now(),
	}

	//todo operation record
	err := device.Delete()
	if err != nil {
		return utils.FailResponse(err.Error(), nil)
	}
	return nil
}
