package service

import (
	"fmt"
	"iot_manager_service/app/dao"
	"iot_manager_service/app/utils"
	"time"
)

var InfoBoardService = new(infoBoardService)

type infoBoardService struct{}

func (s *infoBoardService) Get(id int) (*dao.InfoBoard, *utils.Errors) {
	// 创建查询实例
	device := &dao.InfoBoard{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}
	return device, nil

}

func (s *infoBoardService) CreateOrUpdate(req dao.InfoBoard) *utils.Errors {
	// 创建查询实例
	device := req
	if req.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = "TODO" // todo: 使用登录态
		if device.IsExistedBySN() {
			fmt.Printf("Create GetDeviceID err \n")
			return utils.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
		}
		if err := device.Create(); err != nil {
			fmt.Printf("Create err = %s \n", err.Error())
			return utils.FailResponse(err.Error(), nil)
		}
		return utils.SuccessResponse(utils.Succeeded, nil)
	}
	device.UpdateUser = "TODO" // todo: 使用登录态
	device.UpdateTime = time.Now()
	if device.IsExistedByNameAndCode() {
		return utils.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
	}

	if err := device.Update(); err != nil {
		fmt.Printf("Update err = %s \n", err.Error())
		return utils.FailResponse(err.Error(), nil)
	}

	//todo operation record
	return utils.SuccessResponse(utils.Succeeded, nil)
}
func (s *infoBoardService) List(searchValue string, current, size int) ([]dao.InfoBoard, *utils.Errors) {
	device := dao.InfoBoard{}
	if searchValue != "" {
		device.SN = searchValue
		device.InfoName = searchValue
	}

	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}
	return devices, nil
}
func (s *infoBoardService) Remove(id int) *utils.Errors {
	// 创建查询实例
	device := &dao.InfoBoard{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: "TODO", // todo 使用登录态
		UpdateTime: time.Now(),
	}
	//todo
	// service.transformerService.CountRelation()

	//todo operation record
	err := device.Delete()
	if err != nil {
		return utils.FailResponse(err.Error(), nil)
	}
	return nil
}
