package service

import (
	"fmt"
	"iot_manager_service/app/dao"
	"iot_manager_service/app/model"
	"iot_manager_service/app/utils"
	"time"
)

var SwitchBoxService = new(switchBoxService)

type switchBoxService struct{}

func (s *switchBoxService) Get(id int) (*dao.SwitchBox, *utils.Errors) {
	// 创建查询实例
	device := &dao.SwitchBox{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}
	return device, nil

}

func (s *switchBoxService) CreateOrUpdate(req dao.SwitchBox) *utils.Errors {
	// 创建查询实例
	device := req
	if device.TenantId == "" {
		device.TenantId = "000000" // todo: 使用登录态
	}
	if req.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = "TODO" // todo: 使用登录态
		if device.IsExistedBySN() {
			fmt.Printf("Create GetDeviceID err \n")
			return utils.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
		}
		if err := device.Create(); err != nil {
			fmt.Printf("Create err = %s \n", err.Error())
			return utils.FailResponse(err.Error(), nil)
		}
		return utils.SuccessResponse(utils.Succeeded, nil)
	}
	//更新
	device.UpdateUser = "TODO" // todo: 使用登录态
	device.UpdateTime = time.Now()
	if device.IsExistedByNameAndCode() {
		return utils.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
	}

	if err := device.Update(); err != nil {
		fmt.Printf("Update err = %s \n", err.Error())
		return utils.FailResponse(err.Error(), nil)
	}
	//todo operation record
	return utils.SuccessResponse(utils.Succeeded, nil)
}
func (s *switchBoxService) List(searchValue string, current, size int) ([]dao.SwitchBox, *utils.Errors) {
	device := dao.SwitchBox{}
	if searchValue != "" {
		device.BoxSN = searchValue
		device.BoxName = searchValue
	}

	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}
	return devices, nil
}
func (s *switchBoxService) Remove(id int) *utils.Errors {
	// 创建查询实例
	device := &dao.SwitchBox{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: "TODO", // todo 使用登录态
		UpdateTime: time.Now(),
	}
	//todo
	// service.transformerService.CountRelation()

	//todo operation record
	err := device.Delete()
	if err != nil {
		return utils.FailResponse(err.Error(), nil)
	}
	return nil
}

func (s *switchBoxService) GetList() ([]*dao.SwitchBox, *utils.Errors) {
	var devices []*dao.SwitchBox
	err := utils.Redis.Get(getSwitchBoxListRedisKey("000000")).Scan(&devices)
	if err == nil {
		return devices, nil
	}

	device := &dao.SwitchBox{
		TenantId:  "000000", // todo 使用登录态
		IsDeleted: 0,
	}
	devices, err = device.GetAllDevices()
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}
	_ = utils.Redis.Set(getSwitchBoxListRedisKey("000000"), devices, 0).Err()

	return devices, nil
}

func getSwitchBoxListRedisKey(tenantId string) string {
	return fmt.Sprintf(model.SwitchBoxList, tenantId)
}
