package service

import (
	"fmt"
	"iot_manager_service/app/dao"
	"iot_manager_service/app/model"
	"iot_manager_service/app/utils"
	"time"
)

var TransformerService = new(transformerService)

type transformerService struct{}

func (s *transformerService) Get(id int) (*dao.Transformer, *utils.Errors) {
	// 创建查询实例
	device := &dao.Transformer{
		ID: id,
	}
	err := device.GetDevice()
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}
	return device, nil

}

func (s *transformerService) CreateOrUpdate(req dao.Transformer) *utils.Errors {
	// 创建查询实例
	device := req
	if req.ID == 0 {
		device.CreateTime = time.Now()
		device.CreateUser = "TODO" // todo: 使用登录态
		if device.IsExistedBySN() {
			fmt.Printf("Create GetDeviceID err \n")
			return utils.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
		}
		if err := device.Create(); err != nil {
			fmt.Printf("Create err = %s \n", err.Error())
			return utils.FailResponse(err.Error(), nil)
		}
		return utils.SuccessResponse(utils.Succeeded, nil)
	}
	device.UpdateUser = "TODO" // todo: 使用登录态
	device.UpdateTime = time.Now()
	if device.IsExistedByNameAndCode() {
		return utils.ParamsInvalidResponse("编码不能重复，请重新填写！", nil)
	}

	if err := device.Update(); err != nil {
		fmt.Printf("Update err = %s \n", err.Error())
		return utils.FailResponse(err.Error(), nil)
	}

	//todo operation record
	return utils.SuccessResponse(utils.Succeeded, nil)
}
func (s *transformerService) List(searchValue string, current, size int) ([]dao.Transformer, *utils.Errors) {
	device := dao.Transformer{}
	if searchValue != "" {
		device.TransSN = searchValue
		device.TransName = searchValue
	}

	offset := (current - 1) * size
	limit := size
	devices, err := device.GetDevices(offset, limit)
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}
	return devices, nil
}
func (s *transformerService) Remove(id int) *utils.Errors {
	// 创建查询实例
	device := &dao.Transformer{
		ID:         id,
		IsDeleted:  1,
		UpdateUser: "TODO", // todo 使用登录态
		UpdateTime: time.Now(),
	}
	//todo
	// service.transformerService.CountRelation()

	//todo operation record
	err := device.Delete()
	if err != nil {
		return utils.FailResponse(err.Error(), nil)
	}
	return nil
}

func (s *transformerService) GetList() ([]*dao.Transformer, *utils.Errors) {
	var devices []*dao.Transformer
	err := utils.Redis.Get(getTransformerListRedisKey("000000")).Scan(&devices)
	if err == nil {
		return devices, nil
	}

	device := &dao.Transformer{
		TenantId:  "000000", // todo 使用登录态
		IsDeleted: 0,
	}
	devices, err = device.GetAllDevices()
	if err != nil {
		return nil, utils.FailResponse(err.Error(), nil)
	}
	_ = utils.Redis.Set(getTransformerListRedisKey("000000"), devices, 0).Err()

	return devices, nil
}

func getTransformerListRedisKey(tenantId string) string {
	return fmt.Sprintf(model.TransformerList, tenantId)
}
